@@ -3,7 +3,7 @@ package Sidef {
     use utf8;
     use 5.016;
 
-    our $VERSION = '3.99';
+    our $VERSION = '22.03';
 
     our $SPACES      = 0;    # the current number of indentation spaces
     our $SPACES_INCR = 4;    # the number of indentation spaces
@@ -88,6 +88,7 @@ package Sidef {
 
     sub get_sidef_config_dir {
         my ($self) = @_;
+
         $self->{sidef_config_dir} //= $ENV{SIDEF_CONFIG_DIR}
           || File::Spec->catdir(
                                 $ENV{XDG_CONFIG_DIR}
@@ -104,6 +105,14 @@ package Sidef {
                                 '.config',
                                 'sidef'
                                );
+
+        if (not -d $self->{sidef_config_dir}) {
+            require File::Path;
+            eval { File::Path::make_path($self->{sidef_config_dir}) }
+              or warn "[WARNING] Can't create directory <<$self->{sidef_config_dir}>>: $!";
+        }
+
+        return $self->{sidef_config_dir};
     }
 
     sub get_sidef_vdir {
