@@ -43,7 +43,7 @@ package Sidef::Types::Array::Array {
             $addr{$refaddr} = \@array;
 
             foreach my $item (@$obj) {
-                if (index(ref($item), 'Sidef::') == 0) {
+                if (CORE::index(ref($item), 'Sidef::') == 0) {
                     CORE::push(@array, $item->get_value);
                 }
                 else {
@@ -374,8 +374,13 @@ package Sidef::Types::Array::Array {
 
         my @obj = @$self;
 
+        $num = CORE::int($num);
+        $num > 0 or return undef;
+
         my @array;
-        my $len = CORE::int(scalar(@obj) / CORE::int($num));
+        my $len = CORE::int(scalar(@obj) / $num);
+
+        $len || return undef;
 
         my $i   = 1;
         my $pos = $len;
@@ -1099,6 +1104,12 @@ package Sidef::Types::Array::Array {
         $self->_reduce_by('gcd', Sidef::Types::Number::Number::ZERO, sub { $block->run($_[1]) });
     }
 
+    sub gcud_by {
+        my ($self, $block) = @_;
+        $block //= Sidef::Types::Block::Block::IDENTITY;
+        $self->_reduce_by('gcud', Sidef::Types::Number::Number::ZERO, sub { $block->run($_[1]) });
+    }
+
     sub gcd {
         my ($self, $block) = @_;
 
@@ -1109,6 +1120,16 @@ package Sidef::Types::Array::Array {
         Sidef::Types::Number::Number::gcd(@$self);
     }
 
+    sub gcud {
+        my ($self, $block) = @_;
+
+        if (defined($block)) {
+            goto &gcud_by;
+        }
+
+        Sidef::Types::Number::Number::gcud(@$self);
+    }
+
     sub lcm_by {
         my ($self, $block) = @_;
         $block //= Sidef::Types::Block::Block::IDENTITY;
@@ -2282,18 +2303,51 @@ package Sidef::Types::Array::Array {
             $n = 1;
         }
 
-        state $block = Sidef::Types::Block::Block->new(code => sub { $_[1]->sub($_[0]) });
+        my @diffs = @$self;
 
         foreach my $i (1 .. $n) {
-            $self = $self->map_cons(2, $block);
+            my @tmp;
+            my $prev = shift(@diffs);
+
+            while (@diffs) {
+                my $curr = shift(@diffs);
+                CORE::push(@tmp, $curr->sub($prev));
+                $prev = $curr;
+            }
+            @diffs = @tmp;
         }
 
-        $self;
+        bless \@diffs, ref($self);
     }
 
     *diffs           = \&differences;
     *nth_differences = \&differences;
 
+    sub solve_seq {
+        my ($self, $offset) = @_;
+
+        $offset //= Sidef::Types::Number::Number::ZERO;
+
+        my $poly    = Sidef::Types::Number::Polynomial->new();
+        my $x       = Sidef::Types::Number::Polynomial->new(1 => Sidef::Types::Number::Number::ONE)->sub($offset);
+        my $is_zero = Sidef::Types::Block::Block->new(code => sub { $_[0]->is_zero });
+
+        @$self || return $poly;
+
+        for (my $k = 0 ; ; ++$k) {
+            $poly = $poly->add($x->binomial(Sidef::Types::Number::Number::_set_int($k))->mul($self->[0]));
+            $self = $self->differences;
+            last if $self->all($is_zero);
+        }
+
+        $poly;
+    }
+
+    sub binsplit {
+        my ($self, $block) = @_;
+        Sidef::Types::Number::Number::_binsplit([@$self], $block);
+    }
+
     sub reduce {
         my ($self, $obj, $initial) = @_;
 
@@ -3050,7 +3104,11 @@ package Sidef::Types::Array::Array {
         my @perm;
         my @arr = @$self;
 
-        $n = Sidef::Types::Number::Number->new($n)->int;
+        if (ref($n) ne 'Sidef::Types::Number::Number') {
+            $n = Sidef::Types::Number::Number->new($n);
+        }
+
+        $n = $n->int;
         $n = ref($$n) eq 'Math::GMPz' ? Math::GMPz::Rmpz_init_set($$n) : return undef;
 
         my $sgn = Math::GMPz::Rmpz_sgn($n);
@@ -3080,6 +3138,13 @@ package Sidef::Types::Array::Array {
 
     *nth_perm = \&nth_permutation;
 
+    sub random_permutation {
+        my ($self) = @_;
+        $self->nth_permutation($self->len->factorial->irand);
+    }
+
+    *rand_perm = \&random_permutation;
+
     sub next_permutation {
         my ($self) = @_;
 
@@ -3330,7 +3395,7 @@ package Sidef::Types::Array::Array {
                     push @result, bless [@_];    # don't bless \@_
                 },
                 map { [@$_] } @$self
-                                            );
+            );
 
             return bless \@result;
         }
