@@ -19,6 +19,11 @@ package Sidef::Types::Number::Quadratic {
     sub new {
         my (undef, $x, $y, $w) = @_;
 
+        # Handle evaluation of polynomials
+        if (ref($_[0]) eq __PACKAGE__) {
+            return $_[0]->eval($x);
+        }
+
         $x //= Sidef::Types::Number::Number::ZERO;
         $y //= Sidef::Types::Number::Number::ZERO;
         $w //= Sidef::Types::Number::Number::ONE;
@@ -32,6 +37,11 @@ package Sidef::Types::Number::Quadratic {
 
     *call = \&new;
 
+    sub eval {
+        my ($x, $v) = @_;
+        __PACKAGE__->new($x->{a}->eval($v), $x->{b}->eval($v), $x->{w}->eval($v),);
+    }
+
     sub a {
         $_[0]->{a};
     }
@@ -126,7 +136,7 @@ package Sidef::Types::Number::Quadratic {
 
         # (x + y√d) + (z + w√d) = (x + z) + (y + w)√d
 
-        if (ref($y) eq __PACKAGE__) {
+        if (ref($y) eq __PACKAGE__ and $x->{w} eq $y->{w}) {
             return __PACKAGE__->new($x->{a}->add($y->{a}), $x->{b}->add($y->{b}), $x->{w});
         }
 
@@ -136,7 +146,7 @@ package Sidef::Types::Number::Quadratic {
     sub sub {
         my ($x, $y) = @_;
 
-        if (ref($y) eq __PACKAGE__) {
+        if (ref($y) eq __PACKAGE__ and $x->{w} eq $y->{w}) {
             return __PACKAGE__->new($x->{a}->sub($y->{a}), $x->{b}->sub($y->{b}), $x->{w});
         }
 
@@ -148,7 +158,7 @@ package Sidef::Types::Number::Quadratic {
 
         # (x + y√d) (z + w√d) = (xz+ ywd) + (xw + yz)√d
 
-        if (ref($y) eq __PACKAGE__) {
+        if (ref($y) eq __PACKAGE__ and $x->{w} eq $y->{w}) {
             return __PACKAGE__->new(
 
                 # Quadratic(a*a' + b*b'*w, a*b' + b*a', w)
@@ -156,7 +166,7 @@ package Sidef::Types::Number::Quadratic {
                 $x->{a}->mul($y->{a})->add($x->{b}->mul($y->{b})->mul($x->{w})),
                 $x->{a}->mul($y->{b})->add($x->{b}->mul($y->{a})),
                 $x->{w},
-            );
+                                   );
         }
 
         __PACKAGE__->new($x->{a}->mul($y), $x->{b}->mul($y), $x->{w});
@@ -388,7 +398,7 @@ package Sidef::Types::Number::Quadratic {
             *{__PACKAGE__ . '::' . $method} = sub {
                 my ($x, $y) = @_;
 
-                if (ref($y) eq __PACKAGE__) {
+                if (ref($y) eq __PACKAGE__ and $x->{w} eq $y->{w}) {
                     return __PACKAGE__->new($x->{a}->$method($y->{a}), $x->{b}->$method($y->{b}), $x->{w});
                 }
 
